from datetime import datetime
import json
import os
import shutil
import unittest

import mock
import responses

from cumulusci.core.config import BaseGlobalConfig
from cumulusci.core.config import BaseProjectConfig
from cumulusci.core.config import TaskConfig
from cumulusci.core.keychain import BaseProjectKeychain
from cumulusci.tasks import bulkdata
from cumulusci.tests.util import DummyOrgConfig
from cumulusci.utils import temporary_dir


class TestEpochType(unittest.TestCase):
    def test_process_bind_param(self):
        obj = bulkdata.EpochType()
        dt = datetime(1970, 1, 1, 0, 0, 1)
        result = obj.process_bind_param(dt, None)
        self.assertEquals(1000, result)

    def test_process_result_value(self):
        obj = bulkdata.EpochType()
        result = obj.process_result_value(1000, None)
        self.assertEquals(datetime(1970, 1, 1, 0, 0, 1), result)


BULK_DELETE_QUERY_RESULT = b"Id\n003000000000001".splitlines()
BULK_DELETE_RESPONSE = b'<root xmlns="http://ns"><id>4</id></root>'


def _make_task(task_class, task_config):
    task_config = TaskConfig(task_config)
    global_config = BaseGlobalConfig()
    project_config = BaseProjectConfig(global_config)
    keychain = BaseProjectKeychain(project_config, "")
    project_config.set_keychain(keychain)
    org_config = DummyOrgConfig(
        {"instance_url": "example.com", "access_token": "abc123"}, "test"
    )
    return task_class(project_config, task_config, org_config)


@mock.patch("cumulusci.tasks.bulkdata.time.sleep", mock.Mock())
class TestDeleteData(unittest.TestCase):
    @responses.activate
    def test_run(self):
        api = mock.Mock()
        api.endpoint = "http://api"
        api.jobNS = "http://ns"
        api.create_query_job.return_value = query_job = "1"
        api.query.return_value = query_batch = "2"
        api.is_batch_done.side_effect = [False, True, False, True]
        api.get_all_results_for_query_batch.return_value = [BULK_DELETE_QUERY_RESULT]
        api.create_delete_job.return_value = delete_job = "3"
        api.headers.return_value = {}
        delete_batch = "4"
        responses.add(
            method="POST",
            url="http://api/job/3/batch",
            body=BULK_DELETE_RESPONSE,
            status=200,
        )

        task = _make_task(bulkdata.DeleteData, {"options": {"objects": "Contact"}})
        task.bulk = api

        task()

        api.create_query_job.assert_called_once_with("Contact", contentType="CSV")
        api.query.assert_called_once_with(query_job, "select Id from Contact")
        api.is_batch_done.assert_has_calls(
            [
                mock.call(query_batch, query_job),
                mock.call(query_batch, query_job),
                mock.call(delete_batch, delete_job),
                mock.call(delete_batch, delete_job),
            ]
        )
        api.create_delete_job.assert_called_once_with("Contact", contentType="CSV")
        api.close_job.assert_has_calls([mock.call(query_job), mock.call(delete_job)])


@mock.patch("cumulusci.tasks.bulkdata.time.sleep", mock.Mock())
class TestLoadData(unittest.TestCase):
    @responses.activate
    def test_run(self):
        api = mock.Mock()
        api.endpoint = "http://api"
        api.create_insert_job.side_effect = ["1", "3"]
        api.post_batch.side_effect = ["2", "4"]
        api.is_batch_done.side_effect = [False, True, True]
        api.headers.return_value = {}
        responses.add(
            method="GET",
            url="http://api/job/1/batch/2/result",
            body=b"Id\n1",
            status=200,
        )
        responses.add(
            method="GET",
            url="https://example.com/services/data/vNone/query/?q=SELECT+Id+FROM+RecordType+WHERE+SObjectType%3D%27Account%27AND+DeveloperName+%3D+%27HH_Account%27+LIMIT+1",
            body=json.dumps({"records": [{"Id": "1"}]}),
            status=200,
        )
        responses.add(
            method="GET",
            url="http://api/job/3/batch/4/result",
            body=b"Id\n1",
            status=200,
        )

        base_path = os.path.dirname(__file__)
        db_path = os.path.join(base_path, "testdata.db")
        mapping_path = os.path.join(base_path, "mapping.yml")
        with temporary_dir() as d:
            tmp_db_path = os.path.join(d, "testdata.db")
            shutil.copyfile(db_path, tmp_db_path)

            task = _make_task(
                bulkdata.LoadData,
                {
                    "options": {
                        "database_url": "sqlite:///{}".format(tmp_db_path),
                        "mapping": mapping_path,
                    }
                },
            )
            task.bulk = api
            task()

            contact = task.session.query(task.tables["contacts"]).one()
            self.assertEquals("1", contact.sf_id)
            task.session.close()


HOUSEHOLD_QUERY_RESULT = b"Id\n1".splitlines()
CONTACT_QUERY_RESULT = b"Id,AccountId\n2,1".splitlines()


@mock.patch("cumulusci.tasks.bulkdata.time.sleep", mock.Mock())
class TestQueryData(unittest.TestCase):
    @responses.activate
    def test_run(self):
        api = mock.Mock()
        api.endpoint = "http://api"
        api.create_query_job.side_effect = ["1", "3"]
        api.query.side_effect = ["2", "4"]
        api.get_all_results_for_query_batch.side_effect = [
            [HOUSEHOLD_QUERY_RESULT],
            [CONTACT_QUERY_RESULT],
        ]

        base_path = os.path.dirname(__file__)
        mapping_path = os.path.join(base_path, "mapping.yml")

        task = _make_task(
            bulkdata.QueryData,
            {
                "options": {
                    "database_url": "sqlite://",  # in memory
                    "mapping": mapping_path,
                }
            },
        )
        task.bulk = api
        task()

        contact = task.session.query(task.models["contacts"]).one()
        self.assertEquals("2", contact.sf_id)
        self.assertEquals("1", contact.household_id)



from cumulusci.core.exceptions import RobotTestFailure
from cumulusci.core.tasks import BaseTask
from cumulusci.core.utils import process_list_arg
from cumulusci.tasks.salesforce import BaseSalesforceTask
from robot import run as robot_run
from robot.libdoc import libdoc
from robot.libraries.BuiltIn import BuiltIn
from robot.testdoc import testdoc


class Robot(BaseSalesforceTask):
    task_options = {
        "suites": {
            "description": 'Paths to test case files/directories to be executed similarly as when running the robot command on the command line.  Defaults to "tests" to run all tests in the tests directory',
            "required": True,
        },
        "tests": {
            "description": "Run only tests matching name patterns.  Can be comma separated and use robot wildcards like *"
        },
        "include": {"description": "Includes tests with a given tag"},
        "exclude": {"description": "Excludes tests with a given tag"},
        "vars": {
            "description": "Pass values to override variables in the format VAR1:foo,VAR2:bar"
        },
        "xunit": {"description": "Set an XUnit format output file for test results"},
        "options": {
            "description": "A dictionary of options to robot.run method.  See docs here for format.  NOTE: There is no cci CLI support for this option since it requires a dictionary.  Use this option in the cumulusci.yml when defining custom tasks where you can easily create a dictionary in yaml."
        },
    }

    def _init_options(self, kwargs):
        super(Robot, self)._init_options(kwargs)

        if "tests" in self.options:
            self.options["tests"] = process_list_arg(self.options["tests"])

        if "include" in self.options:
            self.options["include"] = process_list_arg(self.options["include"])

        if "exclude" in self.options:
            self.options["exclude"] = process_list_arg(self.options["exclude"])

        if "vars" in self.options:
            self.options["vars"] = process_list_arg(self.options["vars"])
        else:
            self.options["vars"] = []
        self.options["vars"].append("org:{}".format(self.org_config.name))

        # Initialize options as a dict
        if "options" not in self.options:
            self.options["options"] = {}

    def _run_task(self):
        options = self.options["options"].copy()

        if "tests" in self.options:
            options["test"] = self.options["tests"]
        if "include" in self.options:
            options["include"] = self.options["include"]
        if "exclude" in self.options:
            options["exclude"] = self.options["exclude"]
        if "vars" in self.options:
            options["variable"] = self.options["vars"]
        if "xunit" in self.options:
            options["xunit"] = self.options["xunit"]

        # Inject CumulusCIRobotListener to build the CumulusCI library instance
        # from self.project_config instead of reinitializing CumulusCI's config
        # listener = CumulusCIRobotListener(self.project_config, self.org_config.name)
        # if 'listener' not in options:
        #    options['listener'] = []
        # options['listener'].append(listener)

        num_failed = robot_run(self.options["suites"], **options)
        if num_failed:
            raise RobotTestFailure("{} tests failed".format(num_failed))


class RobotLibDoc(BaseTask):
    task_options = {
        "path": {
            "description": "The path to the robot library to be documented.  Can be a python file or a .robot file.",
            "required": True,
        },
        "output": {
            "description": "The output file where the documentation will be written",
            "required": True,
        },
    }

    def _run_task(self):
        return libdoc(self.options["path"], self.options["output"])


class RobotTestDoc(BaseTask):
    task_options = {
        "path": {
            "description": "The path containing .robot test files",
            "required": True,
        },
        "output": {
            "description": "The output html file where the documentation will be written",
            "required": True,
        },
    }

    def _run_task(self):
        return testdoc(self.options["path"], self.options["output"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from pkgutil import walk_packages

import cumulusci


def find_packages(path=".", prefix=""):
    yield prefix
    prefix = prefix + "."
    for _, name, ispkg in walk_packages(path, prefix):
        if ispkg:
            yield name


with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = [
    "click==7.0",
    "coloredlogs==9.3.1",
    "docutils==0.14",
    "future==0.16.0",
    "github3.py==0.9.6",
    "HiYaPyCo>=0.4.11",
    "lxml==4.2.1",
    "plaintable==0.1.1",
    "pycrypto==2.6.1",
    "PyGithub==1.39",
    "pytz==2018.4",
    "PyYAML==3.13",
    "raven==6.7.0",
    "requests[security]==2.18.4",
    "robotframework==3.0.4",
    "robotframework-seleniumlibrary==3.1.1",
    "rst2ansi==0.1.5",
    "salesforce-bulk==2.0.0",
    "sarge==0.1.4",
    "selenium==3.11.0",
    "simple-salesforce==0.74.2",
    "SQLAlchemy==1.2.7",
    "xmltodict==0.10.2",
]

test_requirements = [
    "coverage==4.1",
    "coveralls==1.2.0",
    "flake8==2.6.0",
    "mock==2.0.0",
    "nose==1.3.7",
    "nose-tap==1.9",
    "testfixtures==6.0.2",
    "tox==2.3.1",
    "responses==0.9.0",
]

setup(
    name="cumulusci",
    version="2.0.13",
    description="Build and release tools for Salesforce developers",
    long_description=readme + "\n\n" + history,
    author="Salesforce.org",
    author_email="jlantz@salesforce.com",
    url="https://github.com/SFDO-Tooling/CumulusCI",
    packages=list(find_packages(cumulusci.__path__, cumulusci.__name__)),
    package_dir={"cumulusci": "cumulusci"},
    entry_points={"console_scripts": ["cci=cumulusci.cli.cci:main"]},
    include_package_data=True,
    install_requires=requirements,
    license="BSD license",
    zip_safe=False,
    keywords="cumulusci",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
    ],
    test_suite="cumulusci.core.tests",
    tests_require=test_requirements,
)
